; 20151224
(defun FLV_FileRead (fnm / lst)
  (if (setq lst (vle-file->list fnm nil))
    (apply
      'strcat
      (cons
        (car lst)
        (mapcar '(lambda (str) (strcat "\n" str)) (cdr lst))
      )
    )
    (progn
      (princ (strcat "\n*** WARNING: File \"" fnm "\" cannot be read *** "))
      nil
    )
  )
)

; 20170921
; (FLV_Group_To_EnameList "NameOfMyGroup")
; Argument(s):
;   Nme: Name of group.
; Return value:
;   List of enames.
(defun FLV_Group_To_EnameList (nme / dict grp)
  (if
    (and
      (setq dict (vle-dictobjname (namedobjdict) "ACAD_GROUP"))
      (setq grp (vle-dictobjname dict nme))
    )
    (vl-remove
      nil
      (mapcar
        '(lambda (sub) (if (= 340 (car sub)) (cdr sub)))
        (entget grp)
      )
    )
  )
)

; 20160826
; (FLV_SelArea (ssget "_X" '((0 . "*POLYLINE") (410 . "Model"))))
; Argument(s):
;   Sel: Selection set or list of enames.
; Return value:
;   Real.
(defun FLV_SelArea (sel)
  (float ; Required in case sel is empty: (apply '+ nil) => 0
    (apply
      '+
      (mapcar
        '(lambda (obj)
          (cond
            ((vl-position (vla-get-objectname obj) '("AcDbCircle" "AcDbRegion"))
              (vla-get-area obj)
            )
            (
              (and
                (vl-position (vla-get-objectname obj) '("AcDb2dPolyline" "AcDbEllipse" "AcDbPolyline" "AcDbSpline"))
                (vlax-curve-isclosed obj)
              )
              (vla-get-area obj)
            )
            (T
              0.0
            )
          )
        )
        (mapcar
          'vlax-ename->vla-object
          (if (listp sel)
            sel
            (vle-selectionset->list sel)
          )
        )
      )
    )
  )
)

; 20160826
; (FLV_SelCount (ssget "_X" '((0 . "LINE") (410 . "Model") (62 . 1))))
; Argument(s):
;   Sel: Selection set or list of enames.
; Return value:
;   Integer.
(defun FLV_SelCount (sel)
  (if (listp sel)
    (length sel)
    (sslength sel)
  )
)

; 20160826
; (FLV_SelPerimeter (ssget "_X" '((0 . "LINE") (410 . "Model") (62 . 1))))
; Argument(s):
;   Sel: Selection set or list of enames.
; Return value:
;   Real.
(defun FLV_SelPerimeter (sel)
  (float ; Required in case sel is empty: (apply '+ nil) => 0
    (apply
      '+
      (mapcar
        '(lambda (obj)
          (cond
            (
              (vl-position
                (vla-get-objectname obj)
                '("AcDb2dPolyline" "AcDb3dPolyline" "AcDbArc" "AcDbCircle" "AcDbEllipse" "AcDbLine" "AcDbPolyline" "AcDbSpline")
              )
              (vle-curve-getperimeter obj)
            )
            ((= (vla-get-objectname obj) "AcDbRegion")
              (vla-get-perimeter obj)
            )
            (T
              0.0
            )
          )
        )
        (mapcar
          'vlax-ename->vla-object
          (if (listp sel)
            sel
            (vle-selectionset->list sel)
          )
        )
      )
    )
  )
)

; 20160826: Function now loads "FLV_Vars.lsp" from 2 locations.
; 20151224
(defun FLV_LoadVars ( / fnm)
  (princ "\nTrying to load Field Lisp Variables from main folder... ")
  (if
    (and
      (setq fnm (findfile "on_doc_load.lsp"))
      (setq fnm (findfile (strcat (vl-string-translate "\\" "/" (vl-filename-directory fnm)) "/FLV_Vars.lsp")))
    )
    (progn
      (load fnm)
      (princ "\nDone! ")
    )
    (princ "\nFile not found! ")
  )
  (princ "\nTrying to load Field Lisp Variables from drawing folder... ")
  (if
    (and
      (= 1 (getvar 'dwgtitled))
      (setq fnm (findfile (strcat (getvar 'dwgprefix) "FLV_Vars.lsp")))
    )
    (progn
      (load fnm)
      (princ "\nDone! ")
    )
    (princ "\nFile not found! ")
  )
)

; 20151224
(defun FLV_ReactorStart ()
  (if (not *FLV_LoadVars_commandReactor*)
    (setq *FLV_LoadVars_commandReactor*
      (vlr-command-reactor
        nil ; No data.
        '((:vlr-commandwillstart . FLV_ReactorCallBack))
      )
    )
  )
)

; 20151224
(defun FLV_ReactorCallBack (rea lst)
  (if
    (vl-position
      (car lst)
      '("EXPORT" "PLOT" "PREVIEW" "PRINT" "QSAVE" "REGEN" "REGENALL" "SAVE" "SAVEAS" "UPDATEFIELD")
    )
    (FLV_LoadVars)
  )
)

; 20151224
(defun c:FLV_Regen ()
  (FLV_LoadVars)
  (vla-regen (vla-get-activedocument (vlax-get-acad-object)) acallviewports)
  (princ)
)

(c:FLV_Regen)
(FLV_ReactorStart)

(princ)
